@@ -16,7 +16,8 @@ where
 {
     pub non_generic: AccountInfo<'info>,
     pub generic: Account<'info, T>,
-    pub const_generic: Loader<'info, FooAccount<N>>,
+
+    pub const_generic: AccountLoader<'info, FooAccount<N>>,
     pub const_generic_loader: AccountLoader<'info, FooAccount<N>>,
     pub associated: Account<'info, Associated<U>>,
 }
